/**
  * =========================================================================
  *  Copyright (C) since 1995 by Leidson CAF & PlanetaMessenger.org
  *
  *  This module is developed and maintained by PlanetaMessenger.org. 
  *  Specs, New and updated versions can be found in 
  *  http://www.planetamessenger.org
  *  If you want contact the Team please send a email to Project Manager
  *  Leidson Campos Alves Ferreira at leidson@planetamessenger.org
  *
  *  Copyright (C) since 1995 by PlanetaMessenger.org
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * ===========================================================================
  */

/**
  *
  * $Id: tui.cpp 58 2011-03-27 01:51:50Z popolony2k $
  * $Author: popolony2k $
  * $Date: 2011-03-27 01:51:50 +0000 (Sun, 27 Mar 2011) $
  * $Revision: 58 $
  * $HeadURL: file:///svn/p/oldskooltech/code/msx/trunk/tools/cpp/sound/mml2msd/tui.cpp $
  */

#include "tui.h"
#include "version.h"
#include <iostream>

/*
 * Extra parms processing.
 */
enum ExtraParms  {
  PARM_NONE = 0,
  PARM_CUSTOM_MML,
  PARM_CUSTOM_MML_FILE_NAME,
  PARM_OUTPUT_FILE_NAME
};

/*
 * Argument options.
 */
#define __FIND_TEMPO_OPTION             "-ft"
#define __VERBOSE_OPTION                "-v"
#define __ADD_CUSTOM_MML_OPTION         "-ac"
#define __ADD_CUSTOM_MML_FILE_OPTION    "-acf"
#define __REMOVE_INSTRUMENTS_OPTION     "-ri"
#define __OUTPUT_FILE_NAME_OPTION       "-of"
#define __HELP_OPTION                   "-h"
#define __REMARKS_OPTION                "-rc"

#define __MSD_FILE_EXTENSION            ".msd"

/*
 * Non optional parameter errors.
 */
#define __ERROR_CUSTOM_MML              "Invalid custom MML parameter."
#define __ERROR_CUSTOM_MML_FILE_NAME    "Invalid custom MML file name parameter."
#define __ERROR_OUTPUT_FILE_NAME        "Invalid output file name parameter."
#define __ERROR_INPUT_FILE_NAME         "Invalid input file name parameter."


// Private declarations
/**
  * Parse all arguments specified by parameter.
  * @param argv The arguments to parse;
  */
void TUI :: ParseArgs( ArgV argv )  {
  
  ExtraParms    parms = PARM_NONE;
  
  for( ArgV :: iterator itItem = argv.begin(); itItem != argv.end(); itItem++ )  {

    string    &strArg = *itItem;

    /*
     * Process the extra parameters.
     */    
    switch( parms )  {
      case PARM_CUSTOM_MML           : strCustomMML = strArg; 
                                       parms = PARM_NONE;
                                       continue;
      case PARM_CUSTOM_MML_FILE_NAME : strCustomMMLFileName = strArg; 
                                       parms = PARM_NONE;
                                       continue;
      case PARM_OUTPUT_FILE_NAME     : strOutputFileName = strArg; 
                                       parms = PARM_NONE;
                                       continue;
    }

    /*
     * -t option.
     */
    if( strArg.compare( __FIND_TEMPO_OPTION ) == 0 )  {
      bFindTempo = true;
      continue;
    }

    /*
     * -v option.
     */
    if( strArg.compare( __VERBOSE_OPTION ) == 0 )  {
      bVerbose = true;
      continue;
    }
    
    /*
     * -ri option.
     */
    if( strArg.compare( __REMOVE_INSTRUMENTS_OPTION ) == 0 )  {
      bRemoveInstruments = true;
      continue;
    }

    /*
     * -h option.
     */
    if( strArg.compare( __HELP_OPTION ) == 0 )  {
      bHelp = true;
      continue;
    }
    
    /*
     * -rc option.
     */
    if( strArg.compare( __REMARKS_OPTION ) == 0 )  {
      bRemarks = false;
      continue;
    }

    /*
     * -ac option.
     */
    if( strArg.compare( __ADD_CUSTOM_MML_OPTION ) == 0 )  {
      parms = PARM_CUSTOM_MML;
      continue;
    }
    
    /*
     * -acf option.
     */
    if( strArg.compare( __ADD_CUSTOM_MML_FILE_OPTION ) == 0 )  {
      parms = PARM_CUSTOM_MML_FILE_NAME;
      continue;
    }
    
    /*
     * -of option.
     */
    if( strArg.compare( __OUTPUT_FILE_NAME_OPTION ) == 0 )  {
      parms = PARM_OUTPUT_FILE_NAME;
      continue;
    }
    
    /*
     * Get the input file name.
     */
    strInputFileName = strArg;
  }
  
  /*
   * Check for non-optional parameters errors.
   */
  switch( parms )  {
    case PARM_CUSTOM_MML           : strLastError = __ERROR_CUSTOM_MML; break;
    case PARM_CUSTOM_MML_FILE_NAME : strLastError = __ERROR_CUSTOM_MML_FILE_NAME; break;
    case PARM_OUTPUT_FILE_NAME     : strLastError = __ERROR_OUTPUT_FILE_NAME; break;
    case PARM_NONE                 : if( strInputFileName.empty() )
                                       strLastError = __ERROR_INPUT_FILE_NAME; 
                                     break;
  }
  
  /*
   * Errors ???
   */
  if( !strLastError.empty() )
    bInvalidArguments = true;
}

// Public declarations
/**
  * Constructor. initialize all class data.
  * @para, argv The program arguments vector to build
  * text user interface;
  */  
TUI :: TUI( ArgV argv )  {

  bFindTempo = false;
  bVerbose   = false;
  bHelp      = false;
  bRemarks   = true;
  bRemoveInstruments = false;
  bInvalidArguments  = false;
  strCustomMML.clear();
  strCustomMMLFileName.clear();
  strOutputFileName.clear();
  strLastError.clear();

  ParseArgs( argv );
}

/**
  * Destructor. finalize all class data.
  */
TUI :: ~TUI( void )  {
  
}

/**
  * Return the flag value for find tempo 
  * option.
  */
bool TUI :: GetFindTempo( void )  {
  
  return bFindTempo;
}

/**
  * Return the flag value for get verbose 
  * option.
  */
bool TUI :: GetVerbose( void )  {

  return bVerbose;
}

/**
  * Return the flag value for remarks
  * option;
  */
bool TUI :: GetRemarks( void )  {
 
  return bRemarks; 
}

/**
  * Return the flag value for remove instruments
  * option.
  */
bool TUI :: GetRemoveInstruments( void )  {
  
  return bRemoveInstruments;
}

/**
  * Return the flag value for help showing.
  */
bool TUI :: GetHelp( void )  {

  return bHelp; 
}

/**
  * Return the invalid arguments flag.
  */
bool TUI :: HasInvalidArguments( void )  {
 
  return bInvalidArguments;
}

/**
  * Return if exist Custom MML parameter in
  * arguments.
  */
bool TUI :: HasCustomMML( void )  {
  
  return !strCustomMML.empty();
}

/**
  * Return if exist Custom MML file parameter in
  * arguments.
  */
bool TUI :: HasCustomMMLFileName( void )  {
  
  return !strCustomMMLFileName.empty();
}

/**
  * Return the custom mml string.
  */
string TUI :: GetCustomMML( void )  {

  return strCustomMML;  
}

/**
  * Return the custom MML file.
  */
string TUI :: GetCustomMMLFileName( void )  {
  
  return strCustomMMLFileName;
}

/**
  * Return the output file name.
  */
string TUI :: GetOutputFileName( void )  {

  return ( strOutputFileName.empty() ? strInputFileName + __MSD_FILE_EXTENSION : strOutputFileName );
}

/**
  * Return the input file name.
  */
string TUI :: GetInputFileName( void )  {

  return strInputFileName;
}

/**
  * Return the last error occurredon parsing
  * of parameters.
  */
string TUI :: GetLastError( void )  {
  
  return strLastError;
}

/**
  * Show program name and version.
  */
void TUI :: ShowHeader( void )  {

  cout << "mml2msd MML Compiler. CopyLeft (c) since 2011 by PlanetaMessenger.org\n";
  cout << "Vrs. " << VERSION;
  cout << "\n\n";
}

/**
  * Show program help with all command line options.
  */
void TUI :: ShowHelp( void )  {
  
  ShowHeader();  
  cout << "Usage mml2msd [-ft][-v][-ac custom_mml][-acf filename][-of filename][-ri][-h] input_file_name\n\n";
  cout << "Optional parameters:\n\n";
  cout << "-ft  Try to find the tempo on source MML and add it to all channels on destination file.\n";
  cout << "-v   Turn on verbose mode.\n";
  cout << "-ac  Add the custom_mml data to beginning of all channels on destination file.\n";
  cout << "-acf Add custom MML to beginning of all channels based on content of file ";
  cout << "specified by parameter filename. See about how to make a custom MML file.\n";
  cout << "-of  Output to file specified by filename.\n";
  cout << "-ri  Generate destination file without the instruments of source MML.\n";
  cout << "-rc  Remove generated comments on destination MML.\n";
  cout << "-h   Show this help screen.\n\n";
  cout << "How to make a custom MML file.\n";
  cout << "If you want to use a custom MML file to each channel of source that you're converting, ";
  cout << "just create a file with the following format:\n\n";
  cout << "FM1=<Custom_MML_command>\n";
  cout << "FM2=<Custom_MML_command>\n";
  cout << "FM3=<Custom_MML_command>\n";
  cout << "FM4=<Custom_MML_command>\n";
  cout << "FM5=<Custom_MML_command>\n";
  cout << "FM6=<Custom_MML_command>\n";
  cout << "FMR=<Custom_MML_command>\n";
  cout << "FM8=<Custom_MML_command>\n";
  cout << "FM9=<Custom_MML_command>\n";
  cout << "PSG1=<Custom_MML_command>\n";
  cout << "PSG2=<Custom_MML_command>\n";
  cout << "PSG3=<Custom_MML_command>\n";
  cout << "SCC1=<Custom_MML_command>\n";
  cout << "SCC2=<Custom_MML_command>\n";
  cout << "SCC3=<Custom_MML_command>\n";
  cout << "SCC4=<Custom_MML_command>\n";
  cout << "SCC5=<Custom_MML_command>\n\n";
  cout << "Where <Custom_MML_command> is the MML that you want add to beginning of each specified channel.\n\n";
  cout << "mml2msd was developed by PopolonY2k for PlanetaMessenger.org.\n";
  cout << "For new versions please visit http://sourceforge.net/projects/oldskooltech/\n";
  cout << "Software information at http://www.popolony2k.com.br\n";
  cout << "Project home at http://www.planetamessenger.org\n";
}
